//boost
#include <boost/archive/archive_exception.hpp>
#include <boost/archive/polymorphic_binary_iarchive.hpp>
#include <boost/archive/polymorphic_binary_oarchive.hpp>

//STL
#include <fstream>
#include <iostream>

//MyLibrary
#include "IMyLibraryClass.h"
#include "IMyLibrarySerializer.h"


int main(void)
{

	//Create a library serializer
	IMyLibrarySerializer* myLibrarySerializer = GetMyLibrarySerializer();

	//Create a library object
	IMyLibraryClass* myLibraryObject = GetMyLibraryClass();
	myLibraryObject->SetData(42);
	printf("Before serializing, data = %d\n", myLibraryObject->GetData());


	////////////////////////////


	//Create an output archiver
	std::ofstream outputStream("tmp.bin", std::ios::out | std::ios::binary);
	boost::archive::polymorphic_binary_oarchive outputArchive(outputStream);


	//Serialize the library object
	try
	{
		myLibrarySerializer->Serialize(outputArchive, myLibraryObject);
		//outputArchive & myLibraryObject;	//<-- This line fails in debug mode.  This is the entire reason for creating the MyLibrarySerializer class.
	}
	catch(boost::archive::archive_exception ex)
	{
		printf("Error writing archive\nCode = %d\nText = %s\n", ex.code, ex.what());
		outputStream.close();
		system("pause");
		return 1;
	}


	//Shutdown the output archiver
	outputStream.close();


	////////////////////////////


	//Create an input archiver
	std::ifstream inputStream("tmp.bin", std::ios::in | std::ios::binary);
	boost::archive::polymorphic_binary_iarchive inputArchive(inputStream);


	//Restore the library object
	IMyLibraryClass* myRestoredObject = 0;
	try
	{
		myLibrarySerializer->Serialize(inputArchive, myRestoredObject);
		//inputArchive & myRestoredObject;	//<-- This method fails in debug mode.  See the "outputArchive & myLibraryObject;" line above.
	}
	catch(boost::archive::archive_exception ex)
	{
		printf("Error reading archive\nCode = %d\nText = %s\n", ex.code, ex.what());
		inputStream.close();
		system("pause");
		return 1;
	}

	//Shutdown the input archiver
	inputStream.close();


	////////////////////////////


	//Test the restored object
	printf("After restoring, data = %d\n", myLibraryObject->GetData());


	////////////////////////////


	//Done
	system("pause");
	return 0;
}
